@@ -1,30 +1,25 @@
 package Apache;
+
 use strict;
+use mod_perl 1.17_01;
 use Exporter ();
 use Apache::Constants qw(OK DECLINED);
-use Apache::SIG ();
+use Apache::Connection ();
+use Apache::Server ();
 
 @Apache::EXPORT_OK = qw(exit warn);
-$Apache::VERSION = "1.25";
 
 *import = \&Exporter::import;
 
-unless(defined &bootstrap) {
-    require DynaLoader;
-    @Apache::ISA = qw(DynaLoader);
-}
-
 if (caller eq "CGI::Apache") {
     #we must die here outside of httpd so CGI::Switch works
-    bootstrap Apache $Apache::VERSION;
+    die unless $ENV{MOD_PERL};
 }
-else {
-    if(exists $ENV{MOD_PERL}) {
-      require Apache::Server unless $mod_perl::UNIMPORT{'server'};
-      require Apache::Connection unless $mod_perl::UNIMPORT{'connection'};
-	bootstrap Apache $Apache::VERSION;
-    }
-    Apache::SIG->set;
+
+{
+    no strict;
+    $VERSION = "1.26";
+    __PACKAGE__->mod_perl::boot($VERSION);
 }
 
 if($ENV{MOD_PERL} && perl_hook("Sections")) {
@@ -43,7 +38,7 @@ sub parse_args {
     my($wantarray,$string) = @_;
     return unless defined $string and $string;
     if(defined $wantarray and $wantarray) {
-	return map { Apache::unescape_url_info($_) } split /[=&]/, $string, -1;
+	return map { Apache::unescape_url_info($_) } split /[=&;]/, $string, -1;
     }
     $string;
 }
@@ -51,7 +46,7 @@ sub parse_args {
 sub content {
     my($r) = @_;
     my $ct = $r->header_in("Content-type") || "";
-    return unless $ct eq "application/x-www-form-urlencoded";
+    return unless $ct =~ m!^application/x-www-form-urlencoded!;
     my $buff;
     $r->read($buff, $r->header_in("Content-length"));
     parse_args(wantarray, $buff);
@@ -219,9 +214,9 @@ the description of the Apache C API at http://www.apache.org/docs/.
 
 The request object holds all the information that the server needs to
 service a request.  Apache B<Perl*Handler>s will be given a reference to the
-request object as parameter and may choose update or use it in various
+request object as parameter and may choose to update or use it in various
 ways.  Most of the methods described below obtain information from or
-updates the request object.
+update the request object.
 The perl version of the request object will be blessed into the B<Apache> 
 package, it is really a C<request_rec*> in disguise.
 
@@ -342,7 +337,7 @@ The number of bytes sent to the client, handy for logging, etc.
 
 =item $r->the_request
 
-The request line send by the client, handy for logging, etc.
+The request line sent by the client, handy for logging, etc.
 
 =item $r->proxyreq
 
@@ -361,10 +356,19 @@ The $r->protocol method will return a string identifying the protocol
 that the client speaks.  Typical values will be "HTTP/1.0" or
 "HTTP/1.1".
 
+=item $r->hostname
+
+Returns the server host name, as set by full URI or Host: header.
+
+=item $r->request_time
+
+Returns the time that the request was made.  The time is the local unix
+time in seconds since the epoch.
+
 =item $r->uri( [$uri] )
 
-The $r->uri method will return the requested URI, optionally changing
-it with the first argument.
+The $r->uri method will return the requested URI minus optional query
+string, optionally changing it with the first argument.
 
 =item $r->filename( [$filename] )
 
@@ -378,7 +382,7 @@ The $r->path_info method will return what is left in the path after the
 I<URI --E<gt> filename> translation, optionally changing it with the first 
 argument if you happen to be doing the translation.
 
-=item $r->args
+=item $r->args( [$query_string] )
 
 The $r->args method will return the contents of the URI I<query
 string>.  When called in a scalar context, the entire string is
@@ -388,6 +392,9 @@ I<value> pairs are returned, i.e. it can be used like this:
    $query = $r->args;
    %in    = $r->args;
 
+$r->args can also be used to set the I<query string>. This can be useful
+when redirecting a POST request.
+
 =item $r->headers_in
 
 The $r->headers_in method will return a %hash of client request
@@ -422,7 +429,7 @@ This method is used to read data from the client,
 looping until it gets all of C<$bytes_to_read> or a timeout happens.
 
 In addition, this method sets a timeout before reading with
-C<$r-E<gt>hard_timeout>
+C<$r-E<gt>hard_timeout>.
 
 =item $r->get_remote_host
 
@@ -472,12 +479,12 @@ this variable.
 =item $c->remote_ip
 
 The dotted decimal representation of the remote client's IP address.
-This is set by then server when the connection record is created so
+This is set by the server when the connection record is created so
 is always defined.
 
 You can also set this value by providing an argument to it. This is
-helpful if your server is behind a squid accelerator proxy which add
-a HTTP_X_FORWARDED_FOR header.
+helpful if your server is behind a squid accelerator proxy which adds
+a X-Forwarded-For header.
 
 =item $c->local_addr
 
@@ -522,10 +529,11 @@ If the configuration directive B<IdentityCheck> is set to off: then
 C<$r-E<gt>get_remote_logname> does nothing and C<$c-E<gt>remote_logname> is
 always undefined.
 
-=item $c->user 
+=item $c->user( [$user] )
 
 If an authentication check was successful, the authentication handler
-caches the user name here.
+caches the user name here. Sets the user name to the optional first
+argument.
 
 =item $c->auth_type
 
@@ -536,6 +544,15 @@ C<$c-E<gt>user>, if any.
 
 Returns true if the client stopped talking to us.
 
+=item $c->fileno( [$direction] )
+
+Returns the client file descriptor. If $direction is 0, the input fd
+is returned. If $direction is not null or ommitted, the output fd is
+returned.
+
+This can be used to detect client disconnect without doing any I/O,
+e.g. using IO::Select.
+
 =back
 
 =back
@@ -553,11 +570,13 @@ Returns the value of a per-directory variable specified by the
 C<PerlSetVar> directive.
 
    # <Location /foo/bar>
-   # SetPerlVar  Key  Value
+   # PerlSetVar  Key  Value
    # </Location>
 
    my $val = $r->dir_config('Key');
 
+Keys are case-insensitive.
+
 Will return a I<HASH> reference blessed into the
 I<Apache::Table> class when called in a scalar context with no
 "key" argument. See I<Apache::Table>.
@@ -646,6 +665,10 @@ Returns true if this is a virtual server.
 
 Returns the wild-carded names for ServerAlias servers. 
 
+=item $s->dir_config( $key )
+
+Alias for Apache::dir_config.
+
 =item $s->warn
 
 Alias for Apache::warn.
@@ -654,6 +677,44 @@ Alias for Apache::warn.
 
 Alias for Apache::log_error.
 
+=item $s->uid
+
+Returns the numeric user id under which the server answers requests.
+This is the value of the User directive.
+
+=item $s->gid
+
+Returns the numeric group id under which the server answers requests.
+This is the value of the Group directive.
+
+=item $r->get_handlers( $hook )
+
+Returns a reference to a list of handlers enabled for $hook. $hook is
+a string representing the phase to handle. The returned list is a list
+of references to the handler subroutines.
+
+	$list = $r->get_handlers( 'PerlHandler' );
+
+=item $r->set_handlers( $hook, [\&handler, ... ] )
+
+Sets the list if handlers to be called for $hook. $hook is a string
+representing the phase to handle. The list of handlers is an anonymous
+array of code references to the handlers to install for this request
+phase. The special list [ \&OK ] can be used to disable a particular
+phase.
+
+	$r->set_handlers( PerlLogHandler => [ \&myhandler1, \&myhandler2 ] );
+	$r->set_handlers( PerlAuthenHandler => [ \&OK ] );
+
+=item $r->push_handlers( $hook, \&handler )
+
+Pushes a new handler to be called for $hook. $hook is a string
+representing the phase to handle. The handler is a reference to a
+subroutine to install for this request phase. This handler will be
+called before any configured handlers.
+
+	$r->push_handlers( PerlHandler => \&footer);
+
 =back
 
 =head1 SETTING UP THE RESPONSE
@@ -695,7 +756,7 @@ for the realm defined by the configuration directive C<AuthName>.
 
 Set the handler for a request.
 Normally set by the configuration directive C<AddHandler>.
-  
+
    $r->handler( "perl-script" );
 
 =item $r->notes( $key, [$value] )
@@ -705,13 +766,24 @@ optionally set the value of a named entry.  This table is used by Apache
 modules to pass messages amongst themselves. Generally if you are
 writing handlers in mod_perl you can use Perl variables for this.
 
-   $r->notes("MY_HANDLER", OK);
+   $r->notes("MY_HANDLER" => OK);
    $val = $r->notes("MY_HANDLER");
 
 Will return a I<HASH> reference blessed into the I<Apache::Table>
 class when called in a scalar context with no "key" argument. This
 requires I<Apache::Table>.
 
+=item $r->pnotes( $key, [$value] )
+
+Like $r->notes, but takes any scalar as an value.
+
+   $r->pnotes("MY_HANDLER" => [qw(one two)]);
+   my $val = $r->pnotes("MY_HANDLER");
+   print $val->[0];     # prints "one"
+
+Advantage over just using a Perl variable is that $r->pnotes gets
+cleaned up after every request.
+
 =item $r->subprocess_env( $key, [$value] )
 
 Return the value of a named entry in the Apache C<subprocess_env>
@@ -956,8 +1028,8 @@ $r->send_http_header().  Example of use:
 
    $r->send_cgi_header(<<EOT);
    Location: /foo/bar
-   Content-type: text/html 
-   
+   Content-type: text/html
+
    EOT
 
 =back
