@@ -1,19 +1,22 @@
 package Apache::StatINC;
-
 use strict;
 
-$Apache::StatINC::VERSION = "1.06";
+$Apache::StatINC::VERSION = "1.07";
 
 my %Stat = ($INC{"Apache/StatINC.pm"} => time);
 
 sub handler {
     my $r = shift;
-    my $do_undef = ref($r) && (lc($r->dir_config("UndefOnReload") || '') eq "on");
+    my $do_undef = ref($r) && ((lc($r->dir_config("StatINC_UndefOnReload") ||
+				   $r->dir_config("UndefOnReload")) || '') eq "on");
     my $DEBUG = ref($r) && (lc($r->dir_config("StatINCDebug") || '') eq "on");
-
+    $DEBUG = $r->dir_config("StatINC_Debug") if ref($r) && $r->dir_config("StatINC_Debug");
+    
     while(my($key,$file) = each %INC) {
 	local $^W = 0;
 	my $mtime = (stat $file)[9];
+	# warn and skip the files with relative paths which can't be locate by applying @INC;
+	warn "Apache::StatINC: Can't locate $file\n",next unless defined $mtime and $mtime;
 	unless(defined $Stat{$file}) { 
 	    $Stat{$file} = $^T;
 	}
@@ -21,16 +24,16 @@ sub handler {
 	    if($do_undef and $key =~ /\.pm$/) {
 		require Apache::Symbol;
 		my $class = Apache::Symbol::file2class($key);
-               $class->Apache::Symbol::undef_functions( undef, 1 );
+		$class->Apache::Symbol::undef_functions( undef, 1 );
 	    }
 	    delete $INC{$key};
 	    require $key;
 	    warn "Apache::StatINC: process $$ reloading $key\n"
-		  if $DEBUG;
+		if $DEBUG > 0;
 	}
 	$Stat{$file} = $mtime;
     }
-
+    
     return 1;
 }
 
@@ -69,11 +72,24 @@ script called 'start_httpd' to start apache, and include a line like this:
 
         PERL5LIB=/usr/local/foo/myperllibs; export PERL5LIB
 
+When you have problems with modules not being reloaded, please refer
+to the following lines in 'perlmodlib':
+
+"Always use B<-w>. Try to C<use strict;> (or C<use strict qw(...);>). 
+Remember that you can add C<no strict qw(...);> to individual blocks 
+of code that need less strictness. Always use B<-w>. Always use B<-w>! 
+Follow the guidelines in the perlstyle(1) manual." 
+
+Warnings when running under mod_perl is enabled with 'PerlWarn On' in
+your httpd.conf.
+
+It will most likely help you to find the problem. Really.
+
 =head1 OPTIONS
 
 =over 4
 
-=item UndefOnReload
+=item StatINC_UndefOnReload
 
 Normally, StatINC will turn of warnings to avoid "Subroutine redefined" 
 warnings when it reloads a file.  However, this does not disable the 
@@ -82,14 +98,16 @@ Perl mandatory warning when re-defining C<constant> subroutines
 B<Apache::Symbol> I<undef_functions> method to avoid these mandatory
 warnings:
 
- PerlSetVar UndefOnReload On
+ PerlSetVar StatINC_UndefOnReload On
 
-=item StatINCDebug
+=item StatINC_Debug
 
 You can make StatINC tell when it reloads a module by setting this
 option to on.
 
- PerlSetVar StatINCDebug On
+ PerlSetVar StatINC_Debug 1
+
+The only used debug level is currently 1. 
 
 =back
 
@@ -99,6 +117,7 @@ mod_perl(3)
 
 =head1 AUTHOR
 
-Doug MacEachern
+Currently maintained by Ask Bjoern Hansen <ask@netcetera.dk>.
+Written by Doug MacEachern.
 
 
