@@ -14,6 +14,7 @@ use Config;
 
 $VERSION = '0.01';
 sub IS_MOD_PERL_BUILD () {-e "../lib/mod_perl.pm"}
+my $Is_Win32 = ($^O eq "MSWin32");
 
 sub new {
     my $class = shift;
@@ -44,6 +45,33 @@ sub new {
     }, $class;
 }
 
+sub mmn_eq {
+    my($class, $dir) = @_;
+    return 1 if $Is_Win32; #just assume, till Apache::src works under win32 
+    my $instsrc;
+    {
+	local @INC = grep { !/blib/ } @INC;
+	my $instdir;
+        for (@INC) { 
+            last if -d ($instdir = "$_/auto/Apache/include"); 
+        } 
+	$instsrc = $class->new(dir => $instdir);
+    }
+    my $targsrc = $class->new($dir ? (dir => $dir) : ()); 
+ 
+    my $inst_mmn = $instsrc->module_magic_number; 
+    my $targ_mmn = $targsrc->module_magic_number; 
+
+    unless ($inst_mmn && $targ_mmn) {
+	return 0;
+    }
+    if ($inst_mmn == $targ_mmn) {
+	return 1;
+    }
+    print "Installed MMN $inst_mmn does not match target $targ_mmn\n";
+    return 0;
+}
+
 sub default_dir {
     eval { require Apache::MyConfig };
     return $@ ? 
@@ -103,10 +131,12 @@ sub module_magic_number {
     my $self = shift;
     my $d = asrc(shift || $self->dir);
 
+    return 0 unless $d;
+
     #return $mcache{$d} if $mcache{$d};
     my $fh;
     for (qw(ap_mmn.h http_config.h)) {
-	last if $fh = FileHandle->new("$d/$_");
+	last if $fh = IO::File->new("$d/$_");
     }
     return 0 unless $fh;
 
@@ -137,6 +167,8 @@ sub httpd_version {
     my($string, $extra, @vers);
 
     while(<$fh>) {
+	next unless /^#define/;
+	s/SERVER_PRODUCT \"/\"Apache/; #1.3.13+
 	next unless s/^#define\s+SERVER_(BASE|)VERSION\s+"(.*)\s*".*/$2/;
 	chomp($string = $_);
 
@@ -160,18 +192,26 @@ sub httpd_version {
     return $version;
 }
 
-my $Is_Win32 = ($^O eq "MSWin32");
+sub find_in_inc {
+    my $name = shift;
+    for (@INC) {
+	my $file;
+	if (-e ($file = "$_/auto/Apache/$name")) {
+	    return $file;
+	}
+    }
+}
+
+#XXX
+#do similar for mod_perl.exp and httpd.exp
 
 sub typemaps {
     my $typemaps = [];
     
-    for (@INC) {
-	my $file;
-	if (-e ($file = "$_/auto/Apache/typemap")) {
-	    push @$typemaps, $file;
-	    last;
-	}
+    if (my $file = find_in_inc("typemap")) {
+	push @$typemaps, $file;
     }
+
     if(IS_MOD_PERL_BUILD) {
 	push @$typemaps, "../Apache/typemap";
     }
@@ -184,10 +224,21 @@ sub inc {
     my $src  = $self->dir;
     my $main = $self->main;
     my $os = $Is_Win32 ? "win32" : "unix";
-    my @inc = ("-I$src", "-I$src/modules/perl", "-I$main");
-    for ("src/regex", "$src/os/$os") {
+    my @inc = (); 
+    for ($src, "$src/modules/perl", $main, "$src/regex", "$src/os/$os") {
 	push @inc, "-I$_" if -d $_;
     }
+    my $ssl_dir = "$src/../ssl/include";
+    unless (-d $ssl_dir) {
+	eval { require Apache::MyConfig };
+	$ssl_dir = "$Apache::MyConfig::Setup{SSL_BASE}/include";
+    }
+    push @inc, "-I$ssl_dir" if -d $ssl_dir;
+    require Apache::MyConfig;
+    if (my $apxs = $Apache::MyConfig::Setup{'APXS'}) {
+	my $ainc = `$apxs -q INCLUDEDIR`;
+	push @inc, "-I$ainc" if -d $ainc;
+    }
     return "@inc";
 }
 
@@ -240,7 +291,7 @@ source code.
 Create an object blessed into the B<Apache::src> class.
 
  my $src = Apache::src->new;
- 
+
 =item dir
 
 Top level directory where source files are located.
