@@ -1,25 +1,31 @@
 package Apache::test;
 
 use strict;
-use vars qw(@EXPORT $USE_THREAD $PERL_DIR);
+use vars qw(@EXPORT $USE_THREAD $USE_SFIO $PERL_DIR);
 use Exporter ();
 use Config;
 use FileHandle ();
 *import = \&Exporter::import;
 
 @EXPORT = qw(test fetch simple_fetch have_module skip_test 
-	     $USE_THREAD $PERL_DIR WIN32 grab run_test); 
+	     $USE_THREAD $USE_SFIO $PERL_DIR WIN32 grab run_test); 
 
 BEGIN { 
     if(not $ENV{MOD_PERL}) {
 	eval { require "net/config.pl"; }; #for 'make test'
 	$PERL_DIR = $net::perldir;
     } 
+    # Validate that the OS knows the name of the server in $net::httpserver     
+    # if 'localhost' is not defined, the tests wouldn't pass
+    (my $hostname) = ($net::httpserver =~ /(.*?):/);
+    warn qq{\n*** [Crucial] You must define "$hostname" (e.g. in /etc/hosts) in order for 'make test' to pass\n}  
+      unless gethostbyname $hostname;
 }
 
 $PERL_DIR = $ENV{PERL_DIR} if exists $ENV{PERL_DIR};
 
 $USE_THREAD = ($Config{extensions} =~ /Thread/) || $Config{usethreads};
+$USE_SFIO = (($Config{'usesfio'} || '') eq 'true');
 
 my $Is_Win32 = ($^O eq "MSWin32");
 sub WIN32 () { $Is_Win32 };
@@ -67,12 +73,32 @@ sub fetch {
 sub simple_fetch {
     my $ua = LWP::UserAgent->new;
     my $url = URI::URL->new("http://$net::httpserver");
-    $url->path(shift);
+    my($path,$q) = split /\?/, shift; 
+    $url->path($path);
+    $url->query($q) if $q;
     my $request = new HTTP::Request('GET', $url);
     my $response = $ua->request($request, undef, undef);   
     $response->is_success;
 }
 
+#even if eval $mod fails, the .pm ends up in %INC
+#so the next eval $mod succeeds, when it shouldnot
+
+my %really_have = (
+   'Apache::Table' => sub { 
+       if ($ENV{MOD_PERL}) {
+	   return Apache::Table->can('TIEHASH');
+       }
+       else {
+	   return $net::callback_hooks{PERL_TABLE_API};
+       }
+   },
+);
+
+for (qw(Apache::Cookie Apache::Request)) {
+    $really_have{$_} = $really_have{'Apache::Table'};
+}
+
 sub have_module {
     my $mod = shift;
     my $v = shift;
@@ -81,8 +107,9 @@ sub have_module {
 	 require Apache;
 	 require Apache::Constants;
     };
+
     eval "require $mod";
-    if($v) {
+    if($v and not $@) {
 	eval { 
 	    local $SIG{__WARN__} = sub {};
 	    $mod->UNIVERSAL::VERSION($v);
@@ -101,7 +128,13 @@ sub have_module {
     elsif($@) {
 	warn "$@\n";
     }
-    print "module $mod is installed\n";
+
+    if (my $cv = $really_have{$mod}) {
+	return 0 unless $cv->();
+    }
+
+    print "module $mod is installed\n" unless $ENV{MOD_PERL};
+    
     return 1;
 }
 
